use super::models;
use std::collections::HashMap;

///
pub struct Api {
  client: reqwest::Client,
  url: String,
  k_secret: String,
  k_username: String,
}

impl std::fmt::Debug for Api {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Api of api.kgon.identity").finish()
  }
}

impl Api {
  ///
  pub fn new(url: &str, k_secret: &str, k_username: &str) -> Api {
    Api {
      client: reqwest::Client::new(),
      url: url.to_string(),
      k_secret: k_secret.to_string(),
      k_username: k_username.to_string(),
    }
  }

  ///
  pub async fn list_members(
    &self,
    data: models::ListMembersRequest,
  ) -> Result<models::ListMembersResponse, models::Error> {
    let mut params = HashMap::new();

    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }

    let res = match self
      .client
      .post(format!("{}/users", self.url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::ListMembersResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }

  ///
  pub async fn save_member(
    &self,
    data: models::SaveMemberRequest,
  ) -> Result<models::SaveMemberResponse, models::Error> {
    let mut params = HashMap::new();

    params.insert("username", data.username);
    params.insert("nickname", data.nickname);
    params.insert("siteUsername", data.site_username);

    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }

    let res = match self
      .client
      .post("https://dev-mw.kgonapi.com/register")
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.k_secret.as_str())
      .header("k-username", self.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(err) => {
        return Err(models::Error { code: 0, msg: None });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::SaveMemberResponse>().await {
        Ok(r) => Ok(r),
        Err(e) => Err(models::Error { code: 0, msg: None }),
      },
      _ => Err(models::Error { code: 0, msg: None }),
    }
  }
}

#[cfg(test)]
mod tests {
  use super::super::models;
  use super::*;

  #[tokio::test]
  async fn find_members() {
    let repository = Api::new("c23d770b873b2ce95747abc57052beb0", "Turbo77");

    let r = repository
      .find_members(models::FindMembersRequest { group_key: None })
      .await
      .expect("");

    println!("members: {:?}", r);
  }

  #[tokio::test]
  async fn save_member() {
    let repository = Api::new("c23d770b873b2ce95747abc57052beb0", "Turbo77");

    let r = repository
      .save_member(models::SaveMemberRequest {
        username: "".to_string(),
        nickname: "".to_string(),
        site_username: "".to_string(),
        group_key: None,
      })
      .await
      .expect("");

    println!("member: {:?}", r);
  }
}
